@@ -14,9 +14,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import re
-
 from oslo_config import cfg
+from oslo_utils import netutils
+from oslo_utils import strutils
 from urllib import parse
 
 try:
@@ -26,8 +26,6 @@ except ImportError:
     redis = None
     sentinel = None
 
-from gnocchi import utils
-
 
 SEP_S = ':'
 SEP = b':'
@@ -120,15 +118,10 @@ OPTS = [
 
 
 def _parse_sentinel(sentinel):
-    # IPv6 (eg. [::1]:6379 )
-    match = re.search(r'^\[(\S+)\]:(\d+)$', sentinel)
-    if match:
-        return (match[1], int(match[2]))
-    # IPv4 or hostname (eg. 127.0.0.1:6379 or localhost:6379)
-    match = re.search(r'^(\S+):(\d+)$', sentinel)
-    if match:
-        return (match[1], int(match[2]))
-    raise ValueError('Malformed sentinel server format')
+    host, port = netutils.parse_host_port(sentinel)
+    if host is None or port is None:
+        raise ValueError('Malformed sentinel server format')
+    return (host, port)
 
 
 def get_client(conf, scripts=None):
@@ -155,7 +148,7 @@ def get_client(conf, scripts=None):
         if a not in options:
             continue
         if a in CLIENT_BOOL_ARGS:
-            v = utils.strtobool(options[a][-1])
+            v = strutils.bool_from_string(options[a][-1])
         elif a in CLIENT_LIST_ARGS:
             v = options[a]
         elif a in CLIENT_INT_ARGS:
