@@ -23,7 +23,6 @@ Create Date: 2017-01-11 16:32:40.421758
 import uuid
 
 from alembic import op
-import six
 import sqlalchemy as sa
 import sqlalchemy_utils
 
@@ -87,6 +86,8 @@ def upgrade():
         if rt.tablename != "generic"
     )
 
+    op.drop_constraint("fk_rh_id_resource_id", "resource_history",
+                       type_="foreignkey")
     op.drop_constraint("fk_metric_resource_id_resource_id", "metric",
                        type_="foreignkey")
     for name, table in resource_type_tablenames.items():
@@ -117,8 +118,6 @@ def upgrade():
 
         new_original_resource_id = resource.original_resource_id.replace(
             '/', '_')
-        if six.PY2:
-            new_original_resource_id = new_original_resource_id.encode('utf-8')
         new_id = sa.literal(uuidtype.process_bind_param(
             str(utils.ResourceUUID(
                 new_original_resource_id, resource.creator)),
@@ -172,6 +171,9 @@ def upgrade():
                           "metric", "resource",
                           ("resource_id",), ("id",),
                           ondelete="SET NULL")
+    op.create_foreign_key("fk_rh_id_resource_id",
+                          "resource_history", "resource",
+                          ("id",), ("id",), ondelete="CASCADE")
 
     for metric in connection.execute(metric_table.select().where(
             metric_table.c.name.like("%/%"))):
