@@ -21,7 +21,9 @@ import logging
 import operator
 import uuid
 
+from collections import abc
 import jsonpatch
+from oslo_utils import strutils
 import pecan
 from pecan import rest
 import pyparsing
@@ -53,10 +55,20 @@ try:
 except ImportError:
     PROMETHEUS_SUPPORTED = False
 
+try:
+    from oslo_db.sqlalchemy import models as sqlalchemy_models
+    SQLALCHEMY_SUPPORTED = True
+except ImportError:
+    SQLALCHEMY_SUPPORTED = False
+
 
 ATTRGETTER_GRANULARITY = operator.attrgetter("granularity")
 LOG = logging.getLogger(__name__)
 
+FLATTEN_DICT_TYPES = ((abc.Mapping, sqlalchemy_models.ModelBase)
+                      if SQLALCHEMY_SUPPORTED
+                      else (abc.Mapping,))
+
 
 def arg_to_list(value):
     if isinstance(value, list):
@@ -90,7 +102,7 @@ def flatten_dict_to_keypairs(d, separator=':'):
     :param separator: symbol between names
     """
     for name, value in sorted(d.items()):
-        if isinstance(value, dict):
+        if isinstance(value, FLATTEN_DICT_TYPES):
             for subname, subvalue in flatten_dict_to_keypairs(value,
                                                               separator):
                 yield ('%s%s%s' % (name, separator, subname), subvalue)
@@ -189,7 +201,7 @@ def get_bool_param(name, params, default='false'):
 def strtobool(varname, v):
     """Convert a string to a boolean."""
     try:
-        return utils.strtobool(v)
+        return strutils.bool_from_string(v, strict=True)
     except ValueError as e:
         abort(400, "Unable to parse `%s': %s" % (varname, str(e)))
 
@@ -261,7 +273,6 @@ class ArchivePolicyController(rest.RestController):
             voluptuous.Optional("back_window"): voluptuous.All(
                 voluptuous.Coerce(int), voluptuous.Range(min=0))
         }))
-        # Validate the data
         try:
             ap_items = [archive_policy.ArchivePolicyItem(**item) for item in
                         body['definition']]
@@ -269,12 +280,36 @@ class ArchivePolicyController(rest.RestController):
             abort(400, str(e))
 
         try:
-            return pecan.request.indexer.update_archive_policy(
+            ap_updated = pecan.request.indexer.update_archive_policy(
                 self.archive_policy, ap_items,
                 back_window=body.get('back_window'))
+
+            self.mark_metric_for_truncation_if_needed(body)
+
+            return ap_updated
         except indexer.UnsupportedArchivePolicyChange as e:
             abort(400, str(e))
 
+    def mark_metric_for_truncation_if_needed(self, body):
+        original_archive_policy = pecan.request.indexer.get_archive_policy(
+            self.archive_policy)
+        new_back_window = body.get('back_window')
+        current_back_window = original_archive_policy.back_window
+        needs_raw_data_truncation = \
+            new_back_window and new_back_window != current_back_window and \
+            new_back_window < current_back_window
+        if needs_raw_data_truncation:
+            LOG.info("Backwindow change in the archive policy. Therefore, "
+                     "we will mark all of its metrics to be updated "
+                     "according to it by the Janitor.")
+            pecan.request.indexer. \
+                update_backwindow_changed_for_metrics_archive_policy(
+                    self.archive_policy)
+        else:
+            LOG.debug("No need to update metrics backwindow change "
+                      "status, because it did not change for this archive "
+                      "policy [%s].", self.archive_policy)
+
     @pecan.expose('json')
     def delete(self):
         # NOTE(jd) I don't think there's any point in fetching and passing the
@@ -674,7 +709,7 @@ class MetricsController(rest.RestController):
             attr_filters.append({"=": {k: v}})
 
         policy_filter = pecan.request.auth_helper.get_metric_policy_filter(
-            pecan.request, "list metric")
+            pecan.request, "list metric", allow_resource_project_id=True)
         resource_policy_filter = (
             pecan.request.auth_helper.get_resource_policy_filter(
                 pecan.request, "list metric", resource_type=None,
@@ -1892,8 +1927,9 @@ class AggregationResourceController(rest.RestController):
 
 
 FillSchema = voluptuous.Schema(
-    voluptuous.Any(voluptuous.Coerce(float), "null", "dropna",
-                   msg="Must be a float, 'dropna' or 'null'"))
+    voluptuous.Any(voluptuous.Coerce(float), "null", "dropna", "ffill", "bfill",
+                   "full_ffill", "full_bfill",
+                   msg="Must be a float, 'dropna', 'null', 'ffill', 'bfill', 'full_ffill' or 'full_bfill'")) # noqa
 
 
 def validate_qs(start=None, stop=None, granularity=None,
